from PyQt5.QtWidgets import QMainWindow, QHBoxLayout, QFileDialog, QMessageBox
from PyQt5 import QtCore
from gui.myapp import Ui_MyApp
from appmodel import AppModel
from appview import AppView
from glcanvas import GLCanvas


class AppController(QMainWindow, Ui_MyApp):
    def __init__(self):
        super().__init__()
        super().setupUi(self)

        # Create model object and view object
        self.model = AppModel()
        self.view = AppView(self.model)

        # Create an OpenGL canvas and associate to the
        # canvas widget
        self.glcanvas = GLCanvas(self, self.view)
        self.glcanvas.setParent(self.canvas)
        horizontalLayout = QHBoxLayout(self.canvas)
        horizontalLayout.setContentsMargins(0, 0, 0, 0)
        horizontalLayout.setSpacing(0)
        horizontalLayout.addWidget(self.glcanvas)

        # Get mouse move events even when no buttons are held down
        self.glcanvas.setMouseTracking(True)

        # Set canvas input focus to activate keyboard events
        self.glcanvas.setFocusPolicy(QtCore.Qt.ClickFocus)

    def openFile(self):
        filename, _ = QFileDialog.getOpenFileName(
            self.centralwidget, 'Open File')

        if filename == '':
            return

        try:
            self.model.loadFile(filename)
        except:
            msg = QMessageBox(self)
            msg.setWindowTitle('Error')
            msg.setText('It was not possible read the file')
            msg.exec()
            return

    def popupMessage(self, _msgText):
        msg = QMessageBox()
        msg.setWindowTitle('Attention')
        msg.setIcon(QMessageBox.Warning)
        msg.setText(_msgText)
        msg.exec_()
