import json
from geometry.pnt3d import Pnt3D


class AppModel:

    def __init__(self):
        # Setup default object coordinates (a hexahedron)
        # Faces: -X, -Y, -Z, +X, +Y, +Z
        # Colors: green, cyan, yellow, green, cyan, yellow
        self.faces = [
            {
                "type": "FACE",
                "ID": 0,
                "points": [
                    [0.0, 0.0, 0.0],
                    [0.0, 0.0, 3.0],
                    [0.0, 2.0, 3.0],
                    [0.0, 2.0, 0.0]
                ],
                "normals": [
                    [-1.0,  0.0,  0.0],
                    [-1.0,  0.0,  0.0],
                    [-1.0,  0.0,  0.0],
                    [-1.0,  0.0,  0.0]
                ],
                "colors": [
                    [0.0, 1.0, 0.0],
                    [0.0, 1.0, 0.0],
                    [0.0, 1.0, 0.0],
                    [0.0, 1.0, 0.0]
                ]
            },
            {
                "type": "FACE",
                "ID": 1,
                "points": [
                    [0.0, 0.0, 0.0],
                    [1.0, 0.0, 0.0],
                    [1.0, 0.0, 3.0],
                    [0.0, 0.0, 3.0]
                ],
                "normals": [
                    [ 0.0, -1.0,  0.0],
                    [ 0.0, -1.0,  0.0],
                    [ 0.0, -1.0,  0.0],
                    [ 0.0, -1.0,  0.0]
                ],
                "colors": [
                    [0.0, 1.0, 1.0],
                    [0.0, 1.0, 1.0],
                    [0.0, 1.0, 1.0],
                    [0.0, 1.0, 1.0]
                ]
            },
            {
                "type": "FACE",
                "ID": 2,
                "points": [
                    [0.0, 2.0, 0.0],
                    [1.0, 2.0, 0.0],
                    [1.0, 0.0, 0.0],
                    [0.0, 0.0, 0.0]
                ],
                "normals": [
                    [ 0.0,  0.0, -1.0],
                    [ 0.0,  0.0, -1.0],
                    [ 0.0,  0.0, -1.0],
                    [ 0.0,  0.0, -1.0]
                ],
                "colors": [
                    [1.0, 1.0, 0.0],
                    [1.0, 1.0, 0.0],
                    [1.0, 1.0, 0.0],
                    [1.0, 1.0, 0.0]
                ]
            },
            {
                "type": "FACE",
                "ID": 3,
                "points": [
                    [1.0, 0.0, 0.0],
                    [1.0, 2.0, 0.0],
                    [1.0, 2.0, 3.0],
                    [1.0, 0.0, 3.0]
                ],
                "normals": [
                    [ 1.0,  0.0,  0.0],
                    [ 1.0,  0.0,  0.0],
                    [ 1.0,  0.0,  0.0],
                    [ 1.0,  0.0,  0.0]
                ],
                "colors": [
                    [0.0, 1.0, 0.0],
                    [0.0, 1.0, 0.0],
                    [0.0, 1.0, 0.0],
                    [0.0, 1.0, 0.0]
                ]
            },
            {
                "type": "FACE",
                "ID": 4,
                "points": [
                    [1.0, 2.0, 0.0],
                    [0.0, 2.0, 0.0],
                    [0.0, 2.0, 3.0],
                    [1.0, 2.0, 3.0]
                ],
                "normals": [
                    [ 0.0,  1.0,  0.0],
                    [ 0.0,  1.0,  0.0],
                    [ 0.0,  1.0,  0.0],
                    [ 0.0,  1.0,  0.0]
                ],
                "colors": [
                    [0.0, 1.0, 1.0],
                    [0.0, 1.0, 1.0],
                    [0.0, 1.0, 1.0],
                    [0.0, 1.0, 1.0]
                ]
            },
            {
                "type": "FACE",
                "ID": 5,
                "points": [
                    [0.0, 0.0, 3.0],
                    [1.0, 0.0, 3.0],
                    [1.0, 2.0, 3.0],
                    [0.0, 2.0, 3.0]
                ],
                "normals": [
                    [ 0.0,  0.0,  1.0],
                    [ 0.0,  0.0,  1.0],
                    [ 0.0,  0.0,  1.0],
                    [ 0.0,  0.0,  1.0]
                ],
                "colors": [
                    [1.0, 1.0, 0.0],
                    [1.0, 1.0, 0.0],
                    [1.0, 1.0, 0.0],
                    [1.0, 1.0, 0.0]
                ]
            }
        ]

    def loadFile(self, _file):
        with open(_file, 'r') as file:
            input = json.load(file)
        self.faces = input['faces']

    def isEmpty(self):
        return self.faces == []

    def getFaces(self):
        return self.faces

    def getFacePts(self, _faceID):
        face_dict = self.faces[_faceID]
        pt_coords = face_dict['points']
        pts = []
        for i in range(0, len(pt_coords)):
            pt = Pnt3D(pt_coords[i][0], pt_coords[i][1], pt_coords[i][2])
            pts.append(pt)
        return pts

    def getFaceNormals(self, _faceID):
        face_dict = self.faces[_faceID]
        if 'normals' in face_dict:
            fnormals = face_dict['normals']
        else:
            fnormals = []
            pts = self.getFacePts(_faceID)
            normal = Pnt3D(0.0, 0.0, 0.0)				
            for i in range(2, len(pts)):
                normal += Pnt3D.crossprodP(pts[0], pts[i-1], pts[i])
            normal = Pnt3D.normalize(normal)
            for i in range(0, len(pts)):
                fnormals.append([normal.getX(), normal.getY(), normal.getZ()])
        return fnormals

    def getFaceColors(self, _faceID):
        face_dict = self.faces[_faceID]
        if 'colors' in face_dict:
            fcolors = face_dict['colors']
        else:
            fcolors = []
            pts = self.getFacePts(_faceID)
            for i in range(0, len(pts)):
                fcolors.append([1.0, 1.0, 0.0])
        return fcolors
