import math


class Pnt3D():

    def __init__(self, _x=None, _y=None, _z=None):
        self.x = _x
        self.y = _y
        self.z = _z

    def setX(self, _x):
        self.x = _x

    def setY(self, _y):
        self.y = _y

    def setZ(self, _z):
        self.z = _z

    def getX(self):
        return self.x

    def getY(self):
        return self.y

    def getZ(self):
        return self.z

    def setCoords(self, _x, _y, _z):
        self.x = _x
        self.y = _y
        self.z = _z

    def getCoords(self):
        return self.x, self.y, self.z

    # Equality test with tolerance (Manhattan distance)
    def equal(p1, p2, tol):
        return abs(p1.x - p2.x) < tol.x and abs(p1.y - p2.y) < tol.y and abs(p1.z - p2.z) < tol.z

    # Equality test without tolerance
    def __eq__(p1, p2):
        return (p1.x == p2.x) and (p1.y == p2.y) and (p1.z == p2.z)

    # Inequality test without tolerance
    def __ne__(p1, p2):
        return not (p1 == p2)

    # Addition +
    def __add__(p1, p2):
        return Pnt3D(p1.x + p2.x, p1.y + p2.y, p1.z + p2.z)

    # Addition +=
    def __iadd__(p1, p2):
        p1 = p1 + p2
        return p1

    # Subtraction -
    def __sub__(p1, p2):
        return Pnt3D(p1.x - p2.x, p1.y - p2.y, p1.z - p2.z)

    # Subtraction -=
    def __isub__(p1, p2):
        p1 = p1 - p2
        return p1

    # Scalar multiplication
    def __mul__(p, s):
        return Pnt3D(p.x * s, p.y * s, p.z * s)

    # Division by scalar
    def __truediv__(p, s):
        if s == 0:
            return Pnt3D(0.0, 0.0, 0.0)
        return Pnt3D(p.x / s, p.y / s, p.z / s)

    # Euclidian distance

    def euclidiandistance(p1, p2):
        return math.sqrt((p1.x - p2.x) * (p1.x - p2.x) +
                         (p1.y - p2.y) * (p1.y - p2.y) +
                         (p1.z - p2.z) * (p1.z - p2.z))

    # Manhattan distance
    def manhattandistance(p1, p2):
        return abs(p1.x - p2.x) + abs(p1.y - p2.y) + abs(p1.z - p2.z)

    # Square of size (vector)
    def sizesquare(p):
        return (p.x * p.x) + (p.y * p.y) + (p.z * p.z)

    # Size (vector)
    def size(p):
        return math.sqrt(Pnt3D.sizesquare(p))

    # Dot product (vector)
    def dotprod(p1, p2):
        return (p1.x * p2.x) + (p1.y * p2.y) + (p1.z * p2.z)

    # Cross product (vector)
    def crossprod(p1, p2):
        return Pnt3D(p1.y * p2.z - p1.z * p2.y,
                     p1.z * p2.x - p1.x * p2.z,
                     p1.x * p2.y - p1.y * p2.x)

    # Cross product (point)
    def crossprodP(p1, p2, p3):
        A = p2 - p1
        B = p3 - p1
        return Pnt3D.crossprod(A, B)

    # Normalize (vector)
    def normalize(p):
        norm = Pnt3D.size(p)
        if norm == 0:
            return Pnt3D(0.0, 0.0, 0.0)
        return Pnt3D(p.x / norm, p.y / norm, p.z / norm)

    # Signed volume of wedge p1-p2-p3-p4
    def volume3d(p1, p2, p3, p4):
        A = p2 - p1
        B = p3 - p1
        C = Pnt3D.crossprod(A, B)
        D = p4 - p1
        return Pnt3D.dotprod(C, D) * 0.5
